import { queue } from '../../../kue';
import TAB from '../../../modules/TAB';
import md5 from 'md5';
import imap from '../../../imap';
import path, { resolve } from 'path';
import fs from 'fs';
import Dashboard from '../../../modules/Dashboard';
import { find } from "string-search"
import { simpleParser } from 'mailparser';
import spamcheck from 'spam-detection'
import domain from 'getdomain';
import { parseDomain } from "parse-domain";
import emlformat from 'eml-format';
import { Gctx } from '../../../modules/Gctx';
import request from 'request'
import parse from 'emailjs-mime-parser'
import { db } from '../../../models';
import config from '../../../config';
import { string } from 'joi';
import ContentChecker from '../../../modules/ContentChecker';
import Audit from '../../../modules/Audit';

import redis from "redis";
const client = redis.createClient();

/**
 * 1. GET Sender Domain, SPF Domain, DKIM domain, Sender IP from received-spf
 * 2. Find the list of IPs from flatten spf records (IPs may contains the range so find the CDR range of the IPs and also, it may contain domain name so find the IPs of that too)
 * 3. Find Dkim signature, from that find domain match with sender domain if matches DKIM is aligned else not aligned
 * 4. If SPF or DKIM is pass then DMARC is pass else fail
 */

import dns from 'dns';
import spfParser from 'spf-parse';
import { SPFValidator } from '../../../modules/spf';
import cidrTools from 'cidr-tools';
import ipRangeCheck from "ip-range-check";
import dkimVerify from 'python-dkim-verify';
import dmarc from 'dmarc-solution';

function fetchSPFRecordForDomainFromTXT(records) {
    // console.log("records",records)
    var spfString = null;
    try {
        if (records == undefined) {
            return "v=spf1 ";
        } else {
            for (var i = 0; i < records.length; i++) {
                if (records[i][0].startsWith('v=spf1')) {
                    var spfRecord = spfParser(records[i][0]);
                    if (spfRecord.valid) {
                        for (var j = 0; j < spfRecord.mechanisms.length; j++) {
                            if (spfRecord.mechanisms[j].prefix == 'v') {
                                spfString = spfRecord.mechanisms[j].prefix + "=" + spfRecord.mechanisms[j].value;
                                continue;
                            }
                            else if (spfRecord.mechanisms[j].type == 'all') {

                                break;
                            }
                            else {
                                if (spfRecord.mechanisms[j].value) spfString = spfString + " " + spfRecord.mechanisms[j].type + ":" + spfRecord.mechanisms[j].value;
                                else spfString = spfString + " " + spfRecord.mechanisms[j].type;

                            }
                        }
                        if (spfString && spfString.includes("all"))
                            break;
                    }
                }
            }
        }
        return spfString;
    } catch (err) {
        console.log(err)
        return "v=spf1 ";
    }
}

function getSPfRecord(domain) {
    // console.log("getSPfRecord",domain)
    return new Promise((resolve) => {
        dns.resolveTxt(domain, async (err, addresses) => {
            console.log('TXT records:', addresses, err)
            resolve(await fetchSPFRecordForDomainFromTXT(addresses))
        })
    });
}

function flattenSpfRecord(record, domain, version) {
    return new Promise((resolve, reject) => {
        var sp = new SPFValidator({ 'domain': record, 'expandIncludes': true })
        sp.getRecords(record)
            .then(r => {
                // console.log("r",r)
                var ips = [];
                if (r[record].ip4 && r[record].ip4.length) ips = cidrTools.merge(r[record].ip4)
                if (r[record].ip6 && r[record].ip6.length) ips = ips.concat(cidrTools.merge(r[record].ip6))
                if (r[record].ext && r[record].ext.length) ips = ips.concat(r[record].ext);
                // console.log("ips",ips);
                return ips;
            })
            .then(r => {
                resolve(r)
            })
            .catch(e => {
                console.log("err", e);
                reject(e)
            })
    })
}

async function checkSpf(senderIp, senderDomain) {
    let spfRecord = await getSPfRecord(senderDomain)
    // console.log("spfRecord is",spfRecord)
    let ips = await flattenSpfRecord(spfRecord)
    var check = ipRangeCheck(senderIp, ips) ? true : ipRangeCheck(senderIp, ips)
    // console.log("check is",check)
    return check ? "Pass" : "Fail";
}

async function checkDkim(header) {
    try {
        const result = await dkimVerify(header);
        console.log("result is", result);
        return result ? "Pass" : "Fail";
    } catch (err) {
        console.RequestError("RequestError while verifying dkim", err);
        return "Fail";
    }
}

function fetchDmarcClass(senderDomain) {
    let aspf = "r";
    let adkim = "r";
    // Fetcher from DNS
    return dmarc.fetch(senderDomain)
        .then(record => {
            // console.log(record);        
            if (record.tags.aspf)
                aspf = record.tags.aspf.value;
            if (record.tags.adkim)
                adkim = record.tags.adkim.value;

            return { aspf, adkim }
        })
        .catch(err => {
            console.log(err);
            return { aspf, adkim }
        });
}

function hackRecords(email) {
    return new Promise((resolve, reject) => {
        request.get({
            url: 'https://haveibeenpwned.com/api/v3/breachedaccount/' + email + '?truncateResponse=false',
            method: 'GET',
            headers: {
                'User-Agent': "API",
                'hibp-api-key': '3441eb667eeb4f3d87eab0925093332e'
            }
        },
            function (RequestError, response, body) {
                if (!RequestError && response.statusCode == 200) {
                    resolve({ body: body });
                } else {
                    resolve({ 'body': body ? body.RestException : null });
                }
            });
    })
}

function makeRequestForOwa(url, accessToken, agent, method, payload) {
    return new Promise((resolve, reject) => {
        let options = {};
        if (payload) {
            options = {
                url: url,
                method: method,
                headers: {
                    'Authorization': accessToken,
                    'User-Agent': agent,
                    'Content-Type': 'application/json; charset=utf-8'
                },
                body: JSON.stringify(payload),
            }
        } else {
            options = {
                url: url,
                method: method,
                headers: {
                    'Authorization': accessToken,
                    'User-Agent': agent,
                    'Content-Type': 'application/json; charset=utf-8'
                },
            }
        }

        console.log("Request option is", options)
        request(options,
            function (RequestError, response, body) {
                // console.log("Response", response);
                // console.log("body", body);
                resolve(body)
            });
    })
}

function makeid() {
    var result = '',
        characters = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789',
        charactersLength = 62;
    for (var i = 0; i < 7; i++) { result += characters.charAt(Math.floor(Math.random() * charactersLength)) }
    return result;
}

function trackIP(ip) {
    return new Promise((resolve, reject) => {
        request.get({
            url: 'https://www.iplocate.io/api/lookup/' + ip,
            method: 'GET',
        },
            function (RequestError, response, body) {
                if (!RequestError && response.statusCode == 200) {
                    body = JSON.parse(body)
                    resolve({ country: body.country, continent: body.continent });
                } else {
                    resolve({ 'body': body ? body.RestException : null });
                }
            });
    })
}

const sendRequestToThreatCop = (body) => {
    console.log("Inside sendRequestToThreatCop", config.threatcop.portal);
    request.post(
        config.threatcop.portal, { json: body },
        function (RequestError, response, body) { });
}

function isJSON(str) {
    if (/^\s*$/.test(str)) return false;
    str = str.replace(/\\(?:["\\\/bfnrt]|u[0-9a-fA-F]{4})/g, '@');
    str = str.replace(/"[^"\\\n\r]*"|true|false|null|-?\d+(?:\.\d*)?(?:[eE][+\-]?\d+)?/g, ']');
    str = str.replace(/(?:^|:|,)(?:\s*\[)+/g, '');
    return (/^[\],:{}\s]*$/).test(str);
}

//key example "prefix*"
function deleteKeysByPattern(key) {
    client.keys(key + '*', function (err, keys) {
        console.log("Keys", keys);
        client.del(keys)
    });
}

export default {
    async create_old(req, res, next) {
        let { header, body, attachments, to, from, token, subject } = req.body;
        console.log("Create", req.body);
        if (req.body.fromJMETER == '1') {
            var filename1 = path.join(appRoot, 'api', 'resources', 'email', 'header.txt');
            header = await fs.readFileSync(filename1, 'utf8');
        }
        // if (body.indexOf("rid=") != -1 || body.indexOf('rid%3D') != -1) {
        //     sendRequestToThreatCop(req.body);
        //     return res.status(200).json({ message: "success" });
        // }
        let filename = md5(header) + '.' + md5((body && body.length) ? body : 'null');
        console.log('Recieved New Report with token', token, 'and filename', filename);
        var domainName = to.split('@');
        domainName = domainName[1].split('>');
        db.aggregate.findOne({ where: { reportedBy: to, senderId: from } })
            .then(r => {
                if (r) {
                    console.log("Database is updating");
                    db.aggregate.update({ nature: 'spam' }, { where: { reportedBy: to, senderId: from } })
                        .then(r => {
                            if (r) return db.mailsReported.findOne({ where: { email: { like: '%' + from + '%' } } })
                            else throw new RequestError("Coudnt update spam in aggreagate table")
                        }).then(r => {
                            if (r) {
                                var hamCount = r.hamCount - 1
                                var spamCount = r.spamCount + 1;
                                // client.del(filename,redis.print);
                                // deleteKeysByPattern(filename.substring(0, filename.indexOf(".")))
                                return db.mailsReported.update({ spamCount: spamCount, hamCount: hamCount }, { where: { id: r.id } })
                            } else return db.mailsReported.create({ email: from, spamCount: 1, hamCount: 0 })
                        }).catch(err => {
                            console.log(err)
                        })
                } else {
                    console.log("New row is inserting");
                    db.aggregate.create({ nature: 'spam', reportedBy: to, senderId: from })
                        .then(r => {
                            if (r) return db.mailsReported.create({ email: from, spamCount: 1, hamCount: 0 })
                            else throw new RequestError("coudnt create in aggreate table ")
                        })
                        .catch(err => {
                            console.log(err);
                        })
                }
                return true;
            })
            .then(r => {
                // console.log("Database work done", r, await new TAB().getFile(filename, { token: token }));
                if (r) return new TAB().getFile(filename, { token: token })
                else throw new RequestRequestError("Your Domain is Not authorized to report Mails")
            })
            .then(reportedEmail => {
                console.log("Reported mail", reportedEmail);
                if (!reportedEmail || !reportedEmail.email) {
                    queue.create('process-request', {
                        header: header,
                        body: body,
                        token: token,
                        to: to,
                        from: from,
                        attachments: attachments,
                        fileName: filename,
                        subject
                    }).save();
                    console.log("Queue created");
                } else if (reportedEmail && reportedEmail.email) {
                    console.log("Queue not created");
                    return reportedEmail;
                } else new RequestRequestError("Unexpected Response..");
            })
            .then(reportedEmail => {
                // client.del(filename, redis.print);
                return res.status(200).json({ reportedEmail: reportedEmail });
            })
            .catch(err => {
                console.log(err);
                next(err);
            })
    },
    async create(req, res, next) {
        let { header, body, attachments, to, from, token, subject } = req.body;
        if (body.indexOf("rid=") != -1 || body.indexOf('rid%3D') != -1) {
            sendRequestToThreatCop(req.body);
            return res.status(200).json({ message: "success" });
        }
        let filename = md5(header) + '.' + md5((body && body.length) ? body : 'null');
        console.log('Recieved New Report with token', token, 'and filename', filename);
        var domainName = to.split('@');
        domainName = domainName[1].split('>');
        db.aggregate.findOne({ where: { reportedBy: to, senderId: from } })
            .then(r => {
                if (r) {
                    console.log("Database is updating");
                    db.aggregate.update({ nature: 'spam' }, { where: { reportedBy: to, senderId: from } })
                        .then(r => {
                            if (r) return db.mailsReported.findOne({ where: { email: { like: '%' + from + '%' } } })
                            else throw new Error("Coudnt update spam in aggreagate table")
                        }).then(r => {
                            if (r) {
                                var hamCount = r.hamCount - 1
                                var spamCount = r.spamCount + 1;
                                return db.mailsReported.update({ spamCount: spamCount, hamCount: hamCount }, { where: { id: r.id } })
                            }
                            else return db.mailsReported.create({ email: from, spamCount: 1, hamCount: 0 })
                        }).catch(err => {
                            console.log(err)
                        })
                }
                else {
                    console.log("New row is inserting");
                    db.aggregate.create({ nature: 'spam', reportedBy: to, senderId: from })
                        .then(r => {
                            if (r) return db.mailsReported.create({ email: from, spamCount: 1, hamCount: 0 })
                            else throw new Error("coudnt create in aggreate table ")
                        })
                        .catch(err => {
                            console.log(err);
                        })
                }
                return true;
            })
            .then(r => {
                // console.log("Database work done", r, await new TAB().getFile(filename, { token: token }));
                if (r) return new TAB().getFile(filename, { token: token })
                else throw new RequestError("Your Domain is Not authorized to report Mails")
            })
            .then(reportedEmail => {
                console.log("Reported mail", reportedEmail);
                if (!reportedEmail || !reportedEmail.email) {
                    queue.create('process-request', {
                        header: header,
                        body: body,
                        token: token,
                        to: to,
                        from: from,
                        attachments: attachments,
                        fileName: filename,
                        subject
                    }).save();
                    console.log("Queue created");
                }
                else if (reportedEmail && reportedEmail.email) {
                    console.log("Queue not created");
                    return reportedEmail;
                } else new RequestError("Unexpected Response..");
            })
            .then(reportedEmail => {
                return res.status(200).json({ reportedEmail: reportedEmail });
            })
            .catch(err => {
                console.log(err);
                next(err);
            })
    },
    // filename.substring(0,filename.indexOf("."))
    async notSpam(req, res, next) {
        var { to, from, header, body, subject } = req.body;
        let filename = md5(header) + '.' + md5((body && body.length) ? body : 'null');
        return db.aggregate.findOne({ where: { reportedBy: to, senderId: from } })
            .then(r => {
                if (r) {
                    db.aggregate.update({ nature: 'ham' }, { where: { reportedBy: to, senderId: from } })
                        .then(r => {
                            if (r) return db.mailsReported.findOne({ where: { email: { like: '%' + from + '%' } } })
                            else throw new RequestError("Coudnt update spam in aggreagate table")
                        }).then(r => {
                            if (r) {
                                var hamCount = r.hamCount + 1
                                var spamCount = r.spamCount - 1;
                                // client.del(filename,redis.print);
                                // deleteKeysByPattern(filename.substring(0, filename.indexOf(".")))
                                return db.mailsReported.update({ spamCount: spamCount, hamCount: hamCount }, { where: { id: r.id } })
                            } else return db.mailsReported.create({ email: from, spamCount: 0, hamCount: 1 })
                        }).catch(err => {
                            throw new RequestError("COunt update in ham table")
                        })
                } else {
                    db.aggregate.create({ nature: 'ham', reportedBy: to, senderId: from })
                        .then(r => {
                            if (r) return db.mailsReported.create({ email: from, spamCount: 0, hamCount: 1 })
                            else throw new RequestError("coudnt create in aggreate table ")
                        })
                        .catch(err => {
                            console.log(err);
                            throw new RequestError("COunt update in ham table")
                        })
                }
                return true;
            })
            .then(r => {
                if (r) return new TAB().notSpam({ to: to, from: from, header: header, subject })
                else throw new RequestError("Coudnt mark for spam!")
            })
            .then(r => {
                res.status(200).json({ success: true })
                client.del(filename, redis.print);
            })
            .catch(err => {
                console.log(err);
                next(err);
            })
    },

    async find(req, res, next) {
        console.log("Reques body for find", req.body)
        let { header, body, token } = req.body;
        let filename = md5(header) + '.' + md5((body && body.length) ? body : 'null');
        console.log('Check New Report with token', token, 'and filename', filename);
        // client.set("key", "value", redis.print);
        client.get(filename, (err, reply) => {
            // console.log("Err,reply", err, reply);
            if (err != null || reply == null) {
                new TAB().getFile(filename, { token: token })
                    .then(reportedEmail => {
                        if (reportedEmail) {
                            client.set(filename, JSON.stringify(reportedEmail), redis.print);
                            return res.status(200).json({ reportedEmail: reportedEmail });
                        }
                        throw new RequestRequestError('File not reported', 404)
                    })
                    .catch(err => {
                        console.log(err);
                        next(err);
                    })
            } else {
                console.log("Else Reponse", reply)
                return res.status(200).json({ reportedEmail: reply });
            }
        });
    },

    async disableImap(req, res, next) {
        console.log("Inside disableImap", JSON.parse(JSON.stringify(req.body)));
        if (req.body.companyCode) {
            imap.close(req.body.companyCode)
            res.status(200).json({ success: true })
        } else throw new RequestRequestError("No company ID found to close for IMAP ")
    },

    async enableImap(req, res, next) {
        if (req.body.companyCode) {
            imap.create(req.body.companyCode)
            res.status(200).json({ success: true })
        } else throw new RequestRequestError("No company ID found to close for IMAP ")
    },

    async parseHeader_old(req, res, next) {
        let filename = md5(req.body.header) + '.' + md5((req.body.body && req.body.length) ? req.body.body : 'null');
        var data = {};
        var settings = {};
        var score = 0;
        var senderDomain;
        var SenderIP = null;
        var dmarcClass = '';//= await fetchDmarcClass(senderDomain);

        if (req.body.fromJMETER == '1' || true) {
            req.body.config = { success: { basic: true } };
            var filename1 = path.join(appRoot, 'api', 'resources', 'email', 'header.txt');
            req.body.header = await fs.readFileSync(filename1, 'utf8');
        } else {
            if (typeof (req.body.config) == 'string') req.body.config = JSON.parse(req.body.config);
        }
        return new Dashboard().checkConfig({ to: req.body.token })
            .then(r => {
                settings = r.data;
                return new Promise((reject, resolve) => {
                    simpleParser(req.body.header, {}, (parsed, err) => {
                        if (err) return reject(err);
                        parsedHeader = parsed;
                        resolve(parsed)
                    });
                }).then(parsedHeader => {
                    if (parsedHeader) {
                        return new Promise((resolve, reject) => {
                            var index = null;
                            for (var i = 0; i < parsedHeader.headerLines.length; i++) {
                                if (parsedHeader.headerLines[i]['key'] == "return-path") {
                                    index = i;
                                }
                            }
                            if (index) {
                                try {
                                    var returnPath = parsedHeader.headerLines[index]['line'].split(':')[1];
                                    var returnPathDomain = domain.hostname(returnPath.split('@')[1]);
                                    if (returnPathDomain.endsWith("&#xD;")) returnPathDomain = domain.hostname(returnPath.split('@')[1].split('&#xD;')[0])
                                    if (returnPathDomain.endsWith(">")) returnPathDomain = domain.hostname(returnPath.split('@')[1].split('>')[0])
                                    senderDomain = domain.hostname(req.body.from.split('@')[1]);
                                    if (senderDomain.endsWith(">")) senderDomain = domain.hostname(req.body.from.split('@')[1].split('>')[0]);
                                    if (senderDomain == returnPathDomain && parseDomain(senderDomain).topLevelDomains[0] == parseDomain(returnPathDomain).topLevelDomains[0]) {
                                        data['spfAlignment'] = "Aligned";
                                    } else {
                                        if (senderDomain && returnPathDomain) {
                                            let spfAlignment = dmarcClass.aspf == "s" ? (senderDomain == returnPathDomain || returnPathDomain == senderDomain) : (senderDomain.indexOf(returnPathDomain) != -1 || returnPathDomain.indexOf(senderDomain) != -1)
                                            data['spfAlignment'] = spfAlignment ? "Aligned" : 'Not Aligned'
                                        } else {
                                            data['spfAlignment'] = 'Not Aligned'
                                        }
                                    }
                                    resolve(parsedHeader)
                                } catch (err) {
                                    console.log(err);
                                    console.log("senderDomain", senderDomain, returnPathDomain)
                                    if (senderDomain && returnPathDomain) {
                                        let spfAlignment = dmarcClass.aspf == "s" ? (senderDomain == returnPathDomain || returnPathDomain == senderDomain) : (senderDomain.indexOf(returnPathDomain) != -1 || returnPathDomain.indexOf(senderDomain) != -1)
                                        data['spfAlignment'] = spfAlignment ? "Aligned" : 'Not Aligned'
                                    } else {
                                        data['spfAlignment'] = 'Not Aligned'
                                    }
                                    resolve(parsedHeader);
                                }
                            } else {
                                resolve(parsedHeader)
                            }
                            if ((req.body.config.success && req.body.config.success.spfalign == 'true') || data['spfAlignment'] == "Aligned") score = score + 10;
                        })
                    } else throw new RequestError("Header Cannot be parsed")
                }).then(async r => {
                    if (r) {
                        var parsedHeader = r
                        return new Promise((resolve, reject) => {
                            var index2 = null;
                            for (var i = 0; i < parsedHeader.headerLines.length; i++) {
                                if (parsedHeader.headerLines[i]['key'] == "dkim-signature") {
                                    index2 = i;
                                }
                            }
                            if (index2) {
                                try {
                                    var dkimAlignmentDomain = domain.hostname(parsedHeader.headerLines[index2]['line'].split('d=')[1].split(';')[0])
                                    if (senderDomain == dkimAlignmentDomain && parseDomain(senderDomain).topLevelDomains[0] == parseDomain(dkimAlignmentDomain).topLevelDomains[0]) {
                                        data['dkimAlignment'] = 'Aligned';
                                    } else {
                                        if (senderDomain && dkimAlignmentDomain) {
                                            let dkimAlignment = dmarcClass.adkim == "s" ? (senderDomain == dkimAlignmentDomain || dkimAlignmentDomain == senderDomain) : (senderDomain.indexOf(dkimAlignmentDomain) != -1 || dkimAlignmentDomain.indexOf(senderDomain) != -1)
                                            data['dkimAlignment'] = dkimAlignment ? "Aligned" : "Not Aligned";
                                        } else {
                                            data['dkimAlignment'] = 'Not Aligned'
                                        }
                                    }
                                    resolve(parsedHeader)
                                } catch {
                                    if (senderDomain && dkimAlignmentDomain) {
                                        let dkimAlignment = dmarcClass.adkim == "s" ? (senderDomain == dkimAlignmentDomain || dkimAlignmentDomain == senderDomain) : (senderDomain.indexOf(dkimAlignmentDomain) != -1 || dkimAlignmentDomain.indexOf(senderDomain) != -1)
                                        data['dkimAlignment'] = dkimAlignment ? "Aligned" : "Not Aligned";
                                    } else {
                                        data['dkimAlignment'] = 'Not Aligned'
                                    }
                                    resolve(parsedHeader)
                                }
                            } else {
                                resolve(parsedHeader)
                            }
                            if (data['dkimAlignment'] == "Aligned") score = score + 10;
                        })
                    } else throw new RequestError("spf alignment check cannot be done")
                }).then(r => {
                    if (r) {
                        var parsedHeader = r
                        return new Promise((resolve, reject) => {
                            var index = null;
                            for (var i = 0; i < parsedHeader.headerLines.length; i++) {
                                if (parsedHeader.headerLines[i]['key'] == "received-spf") {
                                    index = i;
                                }
                            }
                            console.log("----------------------------Index is--------------------", index);
                            if (index) {
                                try {
                                    return find(parsedHeader.headerLines[index]['line'], 'client-ip=')
                                        .then(r => {
                                            if (r.length > 0) {
                                                SenderIP = parsedHeader.headerLines[index]['line'].split('client-ip=')[1].split(';')[0];
                                                console.log("SenderIP", SenderIP)
                                                trackIP(SenderIP)
                                                    .then(r => {
                                                        if (r) {
                                                            data['country'] = r.country;
                                                            data['continent'] = r.continent
                                                        } else throw new RequestError("Cannot find dadta from IP")
                                                    })
                                                    .catch(err => {
                                                        console.log(err);
                                                        throw new RequestError("Cannot find dadta from IP")
                                                    })
                                                resolve(parsedHeader)
                                            } else throw new RequestError("Coundt find Clinet IP")
                                        })
                                        .catch(notFound => {
                                            var index = null;
                                            var ip;
                                            for (var i = 0; i < parsedHeader.headerLines.length; i++) {
                                                if (parsedHeader.headerLines[i]['key'] == "authentication-results") {
                                                    index = i;
                                                }
                                            }
                                            var x = parsedHeader.headerLines[index]['line'].split(" ")
                                            for (var i = 0; i < x.length; i++) {
                                                if (x[i].match(/\b(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\./)) {
                                                    ip = x[i].split(")")[0]
                                                }
                                            }
                                            trackIP(ip)
                                                .then(r => {
                                                    if (r) {
                                                        data['country'] = r.country;
                                                        data['continent'] = r.continent
                                                    } else throw new RequestError("Cannot find dadta from IP")
                                                })
                                                .catch(err => {
                                                    console.log(err);
                                                    throw new RequestError("Cannot find dadta from IP")
                                                })
                                            resolve(parsedHeader)
                                        })
                                } catch {
                                    data['country'] = 'Unknown';
                                    data['continent'] = 'Unknown'
                                    resolve(parsedHeader)
                                }
                            } else {
                                resolve(parsedHeader)
                            }
                        })
                    } else throw new RequestError("dkim  alignment check Caannot be done")
                }).then(parsedHeader => {
                    if (parsedHeader) {
                        var index = null;
                        for (var i = 0; i < parsedHeader.headerLines.length; i++) {
                            if (parsedHeader.headerLines[i]['key'] == "authentication-results") {
                                index = i;
                            }
                        }
                        if (index || parsedHeader.headerLines[index] == undefined) {
                            var spf = "Fail",
                                dkim = "Fail",
                                dmarc = "Fail";
                            return [{ dmarc: dmarc, dkim: dkim, spf: spf, parsedHeader }]
                        } else {
                            var string = parsedHeader.headerLines[index]['line'];
                            var spf = "",
                                dkim = "",
                                dmarc = "";
                            return find(string, 'spf=Pass')
                                .then(async r => {
                                    spf = (r.length) ? "Pass" : await checkSpf(SenderIP, senderDomain)
                                    return find(string, 'dkim=Pass')
                                        .then(async q => {
                                            dkim = (q.length) ? "Pass" : await checkDkim(req.body.header);
                                            // console.log("dkim",dkim);
                                            dkim = (q.length) ? "Pass" : "Fail"
                                            return find(string, 'dmarc=Pass')
                                        }).then(z => {
                                            dmarc = (z.length) ? "Pass" : (spf == "Pass" ? "Pass" : (dkim == "Pass" ? "Pass" : "Fail"))
                                            console.log("spf,dkim,dmarc", spf, dkim, dmarc);
                                            if (spf == 'Pass')
                                                if (req.body.config.success.spf == 'true' || req.body.config.success.basic == true) score = score + 10;
                                            if (dkim == 'Pass')
                                                if (req.body.config.success.dkim == 'true' || req.body.config.success.basic == true) score = score + 10;
                                            if (dmarc == 'Pass')
                                                if (req.body.config.success.dmarc == 'true' || req.body.config.success.basic == true) score = score + 10;
                                            return [dmarc, dkim, spf]
                                        }).then(r => [{ dmarc: dmarc, dkim: dkim, spf: spf, parsedHeader }])
                                })
                        }
                    } else throw new RequestError("Sender IP cannot be parsed from Header");
                }).then(async r => {
                    if (r && settings.content != 'false') {
                        let content = (r[0] && r[0].parsedHeader && r[0].parsedHeader.text ? r[0].parsedHeader.text : req.body.bodyText ? req.body.bodyText : '').replace(/[^\x00-\x7F]/g, "")
                        var hamorspam = '';
                        if (content) {
                            var bodyHamorSpam = r[0].parsedHeader.text ? await new ContentChecker().checkContent({ message: content }) : spamcheck.detect(req.body.bodyText)
                            console.log("bodyHamorSpam", bodyHamorSpam);
                            hamorspam = (bodyHamorSpam && bodyHamorSpam == 'spam') ? "Spam" : "Ham"
                        }
                        data['analytics'] = r
                        data['spam'] = hamorspam
                        if (hamorspam == 'Ham')
                            if (req.body.config.success.content == 'true' || req.body.config.success.basic == true) score = score + 30;
                        return data;
                    } else return true;//throw new RequestError("Coudnt parse SPF DMARC DKIM")
                }).then(r => {
                    if (r) {
                        var senderDomain = req.body.from.split('@')[1]
                        var senderDomainHostname = senderDomain ? domain.hostname(req.body.from.split('@')[1]) : "Unknown";
                        // return data;
                        return new Gctx().checkDomain({ domain: senderDomain, hostname: senderDomainHostname })
                    } else throw new RequestError("Coudnt find spam & ham")
                }).then(r => {
                    if (r) {
                        data['gctxhostname'] = "failed"
                        data['gctxdomain'] = "failed"
                        if (data['gctxhostname'] == "failed")
                            if (req.body.config.success.gctx == 'true' || req.body.config.success.basic == true) score = score + 20;
                        //calculate score and gctx
                        data['score'] = score;
                        return r;
                    } else throw new RequestError("coundnt send request to gctx")
                }).then(r => {
                    if (r) return hackRecords(req.body.to)
                    else throw new RequestError("Coudnt find  body ham or spam")
                }).then(r => {
                    if (r) {
                        return new Promise((resolve, reject) => {
                            if (r.body == null) var hackRecord = []
                            else var hackRecord = JSON.parse(r.body);
                            var hackRecordData = []
                            if (hackRecord.length) {
                                for (var i = 0; i < hackRecord.length; i++) {
                                    hackRecordData.push({ Name: hackRecord[i].Name, BreachDate: hackRecord[i].BreachDate, Description: hackRecord[i].Description })
                                }
                                return resolve(hackRecordData)
                            } else return resolve(hackRecordData);
                        })
                    } else throw new RequestError("Coudnt find hack record")
                }).then(r => {
                    if (r) {
                        data['hackRecords'] = r;
                        return data;
                    } else throw new RequestError("Coudnt parse hack Record")
                }).then(r => {
                    // console.log("r", r);
                    if (r) return new TAB().checkSpamHam({ to: req.body.to, from: req.body.from, header: req.body.header })
                    //return db.aggregate.findOne({where: {reportedBy: req.body.to , senderId: req.body.from}})
                    else throw new RequestError("coudnt put hackRecord in data")
                }).then(r => {
                    // data['spambutton'] = 'spam'
                    if (r.success == "Found") data['spambutton'] = 'spam';
                    else data['spambutton'] = 'ham';
                    return data
                }).then(r => {
                    if (r) return db.mailsReported.findOne({ where: { email: { like: '%' + req.body.from + '%' } } })
                    else throw new RequestError("Coudnt find user spam /ham")
                })
                    .then(r => {
                        if (r) {
                            data['reportedCount'] = r.spamCount;
                            res.status(200).json({ data: data, settings: settings })
                        } else {
                            data['reportedCount'] = 0;
                            res.status(200).json({ data: data, settings: settings })
                        }
                        console.log("If response", data);
                        // client.set(filename, (JSON.stringify({ data: data, settings: settings })), redis.print);
                    }).catch(err => {
                        console.log(err);
                        next(err);
                    })
            })
    },

    async parseHeader(req, res, next) {
        // console.log("Request", req.body.header);
        let filename = md5(req.body.header) + '.' + md5((req.body.body && req.body.length) ? req.body.body : 'null');
        // console.log("filename", filename)
        var data = {};
        var settings = {};
        var score = 0;
        var senderDomain;
        var SenderIP = null;
        var dmarcClass = await fetchDmarcClass(senderDomain);

        return new Dashboard().checkConfig({ to: req.body.token })
            .then(r => {
                // console.log("Cofig data", r);
                settings = r.data;
                return new Promise((reject, resolve) => {
                    simpleParser(req.body.header, {}, (parsed, err) => {
                        if (err) return reject(err);
                        parsedHeader = parsed;
                        resolve(parsed)
                    });
                }).then(parsedHeader => {
                    if (parsedHeader) {
                        return new Promise((resolve, reject) => {
                            var index = null;
                            if (typeof (req.body.config) == 'string') req.body.config = JSON.parse(req.body.config);
                            //spf alignment check
                            for (var i = 0; i < parsedHeader.headerLines.length; i++) {
                                if (parsedHeader.headerLines[i]['key'] == "return-path") {
                                    index = i;
                                }
                            }
                            if (index) {
                                try {
                                    var returnPath = parsedHeader.headerLines[index]['line'].split(':')[1];
                                    var returnPathDomain = domain.hostname(returnPath.split('@')[1]);
                                    if (returnPathDomain.endsWith("&#xD;")) returnPathDomain = domain.hostname(returnPath.split('@')[1].split('&#xD;')[0])
                                    if (returnPathDomain.endsWith(">")) returnPathDomain = domain.hostname(returnPath.split('@')[1].split('>')[0])
                                    senderDomain = domain.hostname(req.body.from.split('@')[1]);
                                    if (senderDomain.endsWith(">")) senderDomain = domain.hostname(req.body.from.split('@')[1].split('>')[0]);
                                    if (senderDomain == returnPathDomain && parseDomain(senderDomain).topLevelDomains[0] == parseDomain(returnPathDomain).topLevelDomains[0]) {
                                        data['spfAlignment'] = "Aligned";
                                    } else {
                                        if (senderDomain && returnPathDomain) {
                                            let spfAlignment = dmarcClass.aspf == "s" ? (senderDomain == returnPathDomain || returnPathDomain == senderDomain) : (senderDomain.indexOf(returnPathDomain) != -1 || returnPathDomain.indexOf(senderDomain) != -1)
                                            data['spfAlignment'] = spfAlignment ? "Aligned" : 'Not Aligned'
                                        } else {
                                            data['spfAlignment'] = 'Not Aligned'
                                        }
                                    }
                                    resolve(parsedHeader)
                                } catch {
                                    console.log("senderDomain", senderDomain, returnPathDomain)
                                    if (senderDomain && returnPathDomain) {
                                        let spfAlignment = dmarcClass.aspf == "s" ? (senderDomain == returnPathDomain || returnPathDomain == senderDomain) : (senderDomain.indexOf(returnPathDomain) != -1 || returnPathDomain.indexOf(senderDomain) != -1)
                                        data['spfAlignment'] = spfAlignment ? "Aligned" : 'Not Aligned'
                                    } else {
                                        data['spfAlignment'] = 'Not Aligned'
                                    }
                                    resolve(parsedHeader);
                                }
                            } else {
                                console.log("else spf align")
                                score = score + 10;
                                resolve(parsedHeader)
                            }
                            if (req.body.config.success.spfalign == 'true' || data['spfAlignment'] == "Aligned") score = score + 10;
                        })
                        // })
                    } else throw new Error("Header Cannot be parsed")
                }).then(async r => {
                    if (r) {
                        var parsedHeader = r
                        return new Promise((resolve, reject) => {
                            var index2 = null;
                            for (var i = 0; i < parsedHeader.headerLines.length; i++) {
                                if (parsedHeader.headerLines[i]['key'] == "dkim-signature") {
                                    index2 = i;
                                }
                            }
                            if (index2 && settings.dkimalign != 'false') {
                                try {
                                    var dkimAlignmentDomain = domain.hostname(parsedHeader.headerLines[index2]['line'].split('d=')[1].split(';')[0])
                                    if (senderDomain == dkimAlignmentDomain && parseDomain(senderDomain).topLevelDomains[0] == parseDomain(dkimAlignmentDomain).topLevelDomains[0]) {
                                        data['dkimAlignment'] = 'Aligned';
                                    } else {
                                        if (senderDomain && dkimAlignmentDomain) {
                                            let dkimAlignment = dmarcClass.adkim == "s" ? (senderDomain == dkimAlignmentDomain || dkimAlignmentDomain == senderDomain) : (senderDomain.indexOf(dkimAlignmentDomain) != -1 || dkimAlignmentDomain.indexOf(senderDomain) != -1)
                                            data['dkimAlignment'] = dkimAlignment ? "Aligned" : "Not Aligned";
                                        } else {
                                            data['dkimAlignment'] = 'Not Aligned'
                                        }
                                    }
                                    resolve(parsedHeader)
                                } catch {
                                    if (senderDomain && dkimAlignmentDomain) {
                                        let dkimAlignment = dmarcClass.adkim == "s" ? (senderDomain == dkimAlignmentDomain || dkimAlignmentDomain == senderDomain) : (senderDomain.indexOf(dkimAlignmentDomain) != -1 || dkimAlignmentDomain.indexOf(senderDomain) != -1)
                                        data['dkimAlignment'] = dkimAlignment ? "Aligned" : "Not Aligned";
                                    } else {
                                        data['dkimAlignment'] = 'Not Aligned'
                                    }
                                    resolve(parsedHeader)
                                }
                            } else {
                                score = score + 10;
                                console.log("else dkim align", score)
                                resolve(parsedHeader)
                            }
                            if (data['dkimAlignment'] == "Aligned") score = score + 10;
                        })
                    } else throw new Error("spf alignment check cannot be done")
                }).then(r => {
                    if (r) {
                        var parsedHeader = r
                        return new Promise((resolve, reject) => {
                            var index = null;
                            for (var i = 0; i < parsedHeader.headerLines.length; i++) {
                                if (parsedHeader.headerLines[i]['key'] == "received-spf") {
                                    index = i;
                                }
                            }
                            console.log("----------------------------Index is--------------------", index);
                            try {
                                return find(parsedHeader.headerLines[index]['line'], 'client-ip=')
                                    .then(r => {
                                        if (r.length > 0) {
                                            SenderIP = parsedHeader.headerLines[index]['line'].split('client-ip=')[1].split(';')[0];
                                            console.log("SenderIP", SenderIP)
                                            trackIP(SenderIP)
                                                .then(r => {
                                                    if (r) {
                                                        data['country'] = r.country;
                                                        data['continent'] = r.continent
                                                    } else throw new Error("Cannot find dadta from IP")
                                                })
                                                .catch(err => {
                                                    console.log(err);
                                                    throw new Error("Cannot find dadta from IP")
                                                })
                                            resolve(parsedHeader)
                                        } else throw new Error("Coundt find Clinet IP")
                                    })
                                    .catch(notFound => {
                                        var index = null;
                                        var ip;
                                        for (var i = 0; i < parsedHeader.headerLines.length; i++) {
                                            if (parsedHeader.headerLines[i]['key'] == "authentication-results") {
                                                index = i;
                                            }
                                        }
                                        var x = parsedHeader.headerLines[index]['line'].split(" ")
                                        for (var i = 0; i < x.length; i++) {
                                            if (x[i].match(/\b(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\./)) {
                                                ip = x[i].split(")")[0]
                                            }
                                        }
                                        trackIP(ip)
                                            .then(r => {
                                                if (r) {
                                                    data['country'] = r.country;
                                                    data['continent'] = r.continent
                                                } else throw new Error("Cannot find dadta from IP")
                                            })
                                            .catch(err => {
                                                console.log(err);
                                                throw new Error("Cannot find dadta from IP")
                                            })
                                        resolve(parsedHeader)
                                    })
                            } catch {
                                data['country'] = 'Unknown';
                                data['continent'] = 'Unknown'
                                resolve(parsedHeader)
                            }
                        })
                    } else throw new Error("dkim  alignment check Caannot be done")
                }).then(parsedHeader => {
                    if (parsedHeader) {
                        var index = null;
                        for (var i = 0; i < parsedHeader.headerLines.length; i++) {
                            if (parsedHeader.headerLines[i]['key'] == "authentication-results") {
                                index = i;
                            }
                        }
                        if (index || parsedHeader.headerLines[index] == undefined) {
                            var spf = "Fail",
                                dkim = "Fail",
                                dmarc = "Fail";
                            return [{ dmarc: dmarc, dkim: dkim, spf: spf, parsedHeader }]
                        } else {
                            var string = parsedHeader.headerLines[index]['line'];
                            var spf = "",
                                dkim = "",
                                dmarc = "";
                            return find(string, 'spf=Pass')
                                .then(async r => {
                                    spf = (r.length) ? "Pass" : await checkSpf(SenderIP, senderDomain)
                                    return find(string, 'dkim=Pass')
                                        .then(async q => {
                                            dkim = (q.length) ? "Pass" : await checkDkim(req.body.header);
                                            // console.log("dkim",dkim);
                                            dkim = (q.length) ? "Pass" : "Fail"
                                            return find(string, 'dmarc=Pass')
                                        }).then(z => {
                                            dmarc = (z.length) ? "Pass" : (spf == "Pass" ? "Pass" : (dkim == "Pass" ? "Pass" : "Fail"))
                                            console.log("spf,dkim,dmarc", spf, dkim, dmarc);
                                            if (spf == 'Pass')
                                                if (req.body.config.success.spf == 'true' || req.body.config.success.basic == true) score = score + 10;
                                            if (dkim == 'Pass')
                                                if (req.body.config.success.dkim == 'true' || req.body.config.success.basic == true) score = score + 10;
                                            if (dmarc == 'Pass')
                                                if (req.body.config.success.dmarc == 'true' || req.body.config.success.basic == true) score = score + 10;
                                            return [dmarc, dkim, spf]
                                        }).then(r => [{ dmarc: dmarc, dkim: dkim, spf: spf, parsedHeader }])
                                })
                        }
                    } else throw new Error("Sender IP cannot be parsed from Header");
                }).then(async r => {
                    if (r) {
                        // let content = (r[0].parsedHeader.text ? r[0].parsedHeader.text : req.body.bodyText).replace(/[^\x00-\x7F]/g, "")
                        // var bodyHamorSpam = r[0].parsedHeader.text ? await new ContentChecker().checkContent({ message: content }) : spamcheck.detect(req.body.bodyText)
                        // console.log("bodyHamorSpam", bodyHamorSpam);
                        // var hamorspam = (bodyHamorSpam && bodyHamorSpam == 'spam') ? "Spam" : "Ham"
                        // data['analytics'] = r
                        // data['spam'] = hamorspam
                        // if (hamorspam == 'Ham')
                        //     if (req.body.config.success.content == 'true' || req.body.config.success.basic == true) score = score + 30;
                        // return data;
                        let content = (r[0] && r[0].parsedHeader && r[0].parsedHeader.text ? r[0].parsedHeader.text : req.body.bodyText ? req.body.bodyText : '').replace(/[^\x00-\x7F]/g, "")
                        var bodyHamorSpam = settings.content != 'false' ? spamcheck.detect(content) : "";
                        var hamorspam = (bodyHamorSpam && bodyHamorSpam == 'spam') ? "Spam" : "Ham"
                        data['analytics'] = r
                        data['spam'] = hamorspam
                        if (hamorspam == 'Ham') if (req.body.config.success.content == 'true' || req.body.config.success.basic == true) score = score + 10;
                        return data;
                    } else throw new Error("Coudnt parse SPF DMARC DKIM")
                }).then(r => {
                    if (r) {
                        var senderDomain = req.body.from.split('@')[1]
                        var senderDomainHostname = senderDomain ? domain.hostname(req.body.from.split('@')[1]) : "Unknown";
                        // return data;
                        if (settings.gctx != 'false')
                            return new Gctx().checkDomain({ domain: senderDomain, hostname: senderDomainHostname })
                        console.log("No gctx scan")
                        return true;
                    } else throw new Error("Coudnt find spam & ham")
                }).then(r => {
                    if (r) {
                        data['gctxhostname'] = "failed"
                        data['gctxdomain'] = "failed"
                        if (data['gctxhostname'] == "failed")
                            if (req.body.config.success.gctx == 'true' || req.body.config.success.basic == true) score = score + 20;
                        //calculate score and gctx
                        data['score'] = score;
                        return r;
                    } else throw new Error("coundnt send request to gctx")
                }).then(r => {
                    if (r && settings.hackbutton != 'false') return hackRecords(req.body.to)
                    else return true;//throw new Error("Coudnt find  body ham or spam")
                }).then(r => {
                    if (r && settings.hackbutton != 'false') {
                        return new Promise((resolve, reject) => {
                            if (r.body == null) var hackRecord = []
                            else var hackRecord = JSON.parse(r.body);
                            var hackRecordData = []
                            if (hackRecord.length) {
                                for (var i = 0; i < hackRecord.length; i++) {
                                    hackRecordData.push({ Name: hackRecord[i].Name, BreachDate: hackRecord[i].BreachDate, Description: hackRecord[i].Description })
                                }
                                return resolve(hackRecordData)
                            } else return resolve(hackRecordData);
                        })
                    } else return true;//throw new Error("Coudnt find hack record")
                }).then(r => {
                    if (r) {
                        data['hackRecords'] = r;
                        return data;
                    } else throw new Error("Coudnt parse hack Record")
                }).then(r => {
                    // console.log("r", r);
                    if (r) return new TAB().checkSpamHam({ to: req.body.to, from: req.body.from, header: req.body.header, subject: req.body.subject })
                    else return { success: '' }//throw new Error("coudnt put hackRecord in data")
                }).then(r => {
                    // data['spambutton'] = 'spam'
                    if (r.success == "Found") data['spambutton'] = 'spam';
                    else data['spambutton'] = 'ham';
                    return data
                }).then(r => {
                    if (r) return db.mailsReported.findOne({ where: { email: { like: '%' + req.body.from + '%' } } })
                    else throw new Error("Coudnt find user spam /ham")
                })
                    .then(r => {
                        if (r) {
                            data['reportedCount'] = r.spamCount;
                            res.status(200).json({ data: data, settings: settings })
                        } else {
                            data['reportedCount'] = 0;
                            res.status(200).json({ data: data, settings: settings })
                        }
                        console.log("If response", data);
                    }).catch(err => {
                        console.log(err);
                        res.status(500).json({ failed: err })
                    })
            })
    },

    async parseHeader_old(req, res, next) {
        console.log("Inside parseHeader", req.body);
        let filename = md5(req.body.header) + '.' + md5((req.body.body && req.body.length) ? req.body.body : 'null');
        var data = {};
        var settings = {};
        var score = 0;
        var senderDomain;
        var SenderIP = null;
        var dmarcClass = '';//= await fetchDmarcClass(senderDomain);

        // if (req.body.fromJMETER == '1' || true) {
        //     req.body.config = { success: { basic: true } };
        //     var filename1 = path.join(appRoot, 'api', 'resources', 'email', 'header.txt');
        //     req.body.header = await fs.readFileSync(filename1, 'utf8');
        // } else {
        if (typeof (req.body.config) == 'string') req.body.config = JSON.parse(req.body.config);
        // }
        return new Dashboard().checkConfig({ to: req.body.token })
            .then(r => {
                settings = r.data;
                console.log("req.body.config", settings)
                return new Promise((reject, resolve) => {
                    simpleParser(req.body.header, {}, (parsed, err) => {
                        if (err) return reject(err);
                        parsedHeader = parsed;
                        resolve(parsed)
                    });
                }).then(parsedHeader => {
                    if (parsedHeader) {
                        return new Promise((resolve, reject) => {
                            var index = null;
                            for (var i = 0; i < parsedHeader.headerLines.length; i++) {
                                if (parsedHeader.headerLines[i]['key'] == "return-path") {
                                    index = i;
                                }
                            }
                            if (index) {
                                try {
                                    var returnPath = parsedHeader.headerLines[index]['line'].split(':')[1];
                                    var returnPathDomain = domain.hostname(returnPath.split('@')[1]);
                                    if (returnPathDomain.endsWith("&#xD;")) returnPathDomain = domain.hostname(returnPath.split('@')[1].split('&#xD;')[0])
                                    if (returnPathDomain.endsWith(">")) returnPathDomain = domain.hostname(returnPath.split('@')[1].split('>')[0])
                                    senderDomain = domain.hostname(req.body.from.split('@')[1]);
                                    if (senderDomain.endsWith(">")) senderDomain = domain.hostname(req.body.from.split('@')[1].split('>')[0]);
                                    if (senderDomain == returnPathDomain && parseDomain(senderDomain).topLevelDomains[0] == parseDomain(returnPathDomain).topLevelDomains[0]) {
                                        data['spfAlignment'] = "Aligned";
                                    } else {
                                        if (senderDomain && returnPathDomain) {
                                            let spfAlignment = dmarcClass.aspf == "s" ? (senderDomain == returnPathDomain || returnPathDomain == senderDomain) : (senderDomain.indexOf(returnPathDomain) != -1 || returnPathDomain.indexOf(senderDomain) != -1)
                                            data['spfAlignment'] = spfAlignment ? "Aligned" : 'Not Aligned'
                                        } else {
                                            data['spfAlignment'] = 'Not Aligned'
                                        }
                                    }
                                    resolve(parsedHeader)
                                } catch (err) {
                                    console.log(err);
                                    console.log("senderDomain", senderDomain, returnPathDomain)
                                    if (senderDomain && returnPathDomain) {
                                        let spfAlignment = dmarcClass.aspf == "s" ? (senderDomain == returnPathDomain || returnPathDomain == senderDomain) : (senderDomain.indexOf(returnPathDomain) != -1 || returnPathDomain.indexOf(senderDomain) != -1)
                                        data['spfAlignment'] = spfAlignment ? "Aligned" : 'Not Aligned'
                                    } else {
                                        data['spfAlignment'] = 'Not Aligned'
                                    }
                                    resolve(parsedHeader);
                                }
                            } else {
                                console.log("else spf align")
                                score = score + 10;
                                resolve(parsedHeader)
                            }
                            if ((req.body.config.success && req.body.config.success.spfalign == 'true') || data['spfAlignment'] == "Aligned") score = score + 10;
                        })
                    } else throw new RequestError("Header Cannot be parsed")
                }).then(async r => {
                    if (r) {
                        var parsedHeader = r
                        return new Promise((resolve, reject) => {
                            var index2 = null;
                            for (var i = 0; i < parsedHeader.headerLines.length; i++) {
                                if (parsedHeader.headerLines[i]['key'] == "dkim-signature") {
                                    index2 = i;
                                }
                            }
                            if (index2 && settings.dkimalign != 'false') {
                                try {
                                    var dkimAlignmentDomain = domain.hostname(parsedHeader.headerLines[index2]['line'].split('d=')[1].split(';')[0])
                                    if (senderDomain == dkimAlignmentDomain && parseDomain(senderDomain).topLevelDomains[0] == parseDomain(dkimAlignmentDomain).topLevelDomains[0]) {
                                        data['dkimAlignment'] = 'Aligned';
                                    } else {
                                        if (senderDomain && dkimAlignmentDomain) {
                                            let dkimAlignment = dmarcClass.adkim == "s" ? (senderDomain == dkimAlignmentDomain || dkimAlignmentDomain == senderDomain) : (senderDomain.indexOf(dkimAlignmentDomain) != -1 || dkimAlignmentDomain.indexOf(senderDomain) != -1)
                                            data['dkimAlignment'] = dkimAlignment ? "Aligned" : "Not Aligned";
                                        } else {
                                            data['dkimAlignment'] = 'Not Aligned'
                                        }
                                    }
                                    resolve(parsedHeader)
                                } catch {
                                    if (senderDomain && dkimAlignmentDomain) {
                                        let dkimAlignment = dmarcClass.adkim == "s" ? (senderDomain == dkimAlignmentDomain || dkimAlignmentDomain == senderDomain) : (senderDomain.indexOf(dkimAlignmentDomain) != -1 || dkimAlignmentDomain.indexOf(senderDomain) != -1)
                                        data['dkimAlignment'] = dkimAlignment ? "Aligned" : "Not Aligned";
                                    } else {
                                        data['dkimAlignment'] = 'Not Aligned'
                                    }
                                    resolve(parsedHeader)
                                }
                            } else {
                                score = score + 10;
                                console.log("else dkim align", score)
                                resolve(parsedHeader)
                            }
                            if (data['dkimAlignment'] == "Aligned") score = score + 10;
                        })
                    } else throw new RequestError("spf alignment check cannot be done")
                }).then(r => {
                    if (r) {
                        var parsedHeader = r
                        return new Promise((resolve, reject) => {
                            var index = null;
                            for (var i = 0; i < parsedHeader.headerLines.length; i++) {
                                if (parsedHeader.headerLines[i]['key'] == "received-spf") {
                                    index = i;
                                }
                            }
                            console.log("----------------------------Index is--------------------", index);
                            if (index) {
                                try {
                                    return find(parsedHeader.headerLines[index]['line'], 'client-ip=')
                                        .then(r => {
                                            if (r.length > 0) {
                                                SenderIP = parsedHeader.headerLines[index]['line'].split('client-ip=')[1].split(';')[0];
                                                console.log("SenderIP", SenderIP)
                                                trackIP(SenderIP)
                                                    .then(r => {
                                                        if (r) {
                                                            data['country'] = r.country;
                                                            data['continent'] = r.continent
                                                        } else throw new RequestError("Cannot find dadta from IP")
                                                    })
                                                    .catch(err => {
                                                        console.log(err);
                                                        throw new RequestError("Cannot find dadta from IP")
                                                    })
                                                resolve(parsedHeader)
                                            } else throw new RequestError("Coundt find Clinet IP")
                                        })
                                        .catch(notFound => {
                                            var index = null;
                                            var ip;
                                            for (var i = 0; i < parsedHeader.headerLines.length; i++) {
                                                if (parsedHeader.headerLines[i]['key'] == "authentication-results") {
                                                    index = i;
                                                }
                                            }
                                            var x = parsedHeader.headerLines[index]['line'].split(" ")
                                            for (var i = 0; i < x.length; i++) {
                                                if (x[i].match(/\b(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\./)) {
                                                    ip = x[i].split(")")[0]
                                                }
                                            }
                                            trackIP(ip)
                                                .then(r => {
                                                    if (r) {
                                                        data['country'] = r.country;
                                                        data['continent'] = r.continent
                                                    } else throw new RequestError("Cannot find dadta from IP")
                                                })
                                                .catch(err => {
                                                    console.log(err);
                                                    throw new RequestError("Cannot find dadta from IP")
                                                })
                                            resolve(parsedHeader)
                                        })
                                } catch {
                                    data['country'] = 'Unknown';
                                    data['continent'] = 'Unknown'
                                    resolve(parsedHeader)
                                }
                            } else {
                                resolve(parsedHeader)
                            }
                        })
                    } else throw new RequestError("dkim  alignment check Caannot be done")
                }).then(parsedHeader => {
                    if (parsedHeader) {
                        var index = null;
                        for (var i = 0; i < parsedHeader.headerLines.length; i++) {
                            if (parsedHeader.headerLines[i]['key'] == "authentication-results") {
                                index = i;
                            }
                        }
                        if (index || parsedHeader.headerLines[index] == undefined) {
                            var spf = "Fail",
                                dkim = "Fail",
                                dmarc = "Fail";
                            return [{ dmarc: dmarc, dkim: dkim, spf: spf, parsedHeader }]
                        } else {
                            var string = parsedHeader.headerLines[index]['line'];
                            var spf = "",
                                dkim = "",
                                dmarc = "";
                            return find(string, 'spf=Pass')
                                .then(async r => {
                                    spf = (r.length) ? "Pass" : await checkSpf(SenderIP, senderDomain)
                                    return find(string, 'dkim=Pass')
                                        .then(async q => {
                                            dkim = (q.length) ? "Pass" : await checkDkim(req.body.header);
                                            // console.log("dkim",dkim);
                                            dkim = (q.length) ? "Pass" : "Fail"
                                            return find(string, 'dmarc=Pass')
                                        }).then(z => {
                                            dmarc = (z.length) ? "Pass" : (spf == "Pass" ? "Pass" : (dkim == "Pass" ? "Pass" : "Fail"))
                                            console.log("spf,dkim,dmarc", spf, dkim, dmarc);
                                            if (spf == 'Pass')
                                                if (req.body.config.success.spf == 'true' || req.body.config.success.basic == true) score = score + 10;
                                            if (dkim == 'Pass')
                                                if (req.body.config.success.dkim == 'true' || req.body.config.success.basic == true) score = score + 10;
                                            if (dmarc == 'Pass')
                                                if (req.body.config.success.dmarc == 'true' || req.body.config.success.basic == true) score = score + 10;
                                            return [dmarc, dkim, spf]
                                        }).then(r => [{ dmarc: dmarc, dkim: dkim, spf: spf, parsedHeader }])
                                })
                        }
                    } else throw new RequestError("Sender IP cannot be parsed from Header");
                }).then(async r => {
                    if (r) {
                        let content = (r[0] && r[0].parsedHeader && r[0].parsedHeader.text ? r[0].parsedHeader.text : req.body.bodyText ? req.body.bodyText : '').replace(/[^\x00-\x7F]/g, "")
                        var hamorspam = '';
                        if (content && settings.content != 'false') {
                            var bodyHamorSpam = r[0].parsedHeader.text ? await new ContentChecker().checkContent({ message: content }) : spamcheck.detect(req.body.bodyText)
                            console.log("bodyHamorSpam", bodyHamorSpam);
                            hamorspam = (bodyHamorSpam && bodyHamorSpam == 'spam') ? "Spam" : "Ham"
                        }
                        data['analytics'] = r
                        data['spam'] = hamorspam
                        if (hamorspam == 'Ham')
                            if (req.body.config.success.content == 'true' || req.body.config.success.basic == true) score = score + 30;
                        return data;
                    } else throw new RequestError("Coudnt parse SPF DMARC DKIM")
                }).then(r => {
                    if (r) {
                        var senderDomain = req.body.from.split('@')[1]
                        var senderDomainHostname = senderDomain ? domain.hostname(req.body.from.split('@')[1]) : "Unknown";
                        // return data;
                        if (settings.gctx != 'false')
                            return new Gctx().checkDomain({ domain: senderDomain, hostname: senderDomainHostname })
                        console.log("No gctx scan")
                        return true;
                    } else throw new RequestError("Coudnt find spam & ham")
                }).then(r => {
                    if (r) {
                        data['gctxhostname'] = "failed"
                        data['gctxdomain'] = "failed"
                        if (data['gctxhostname'] == "failed")
                            if (req.body.config.success.gctx == 'true' || req.body.config.success.basic == true) score = score + 20;
                        //calculate score and gctx
                        data['score'] = score;
                        return r;
                    } else throw new RequestError("coundnt send request to gctx")
                }).then(r => {
                    if (r && settings.hackbutton != 'false') return hackRecords(req.body.to)
                    else return true;//throw new RequestError("Coudnt find  body ham or spam")
                }).then(r => {
                    if (r && settings.hackbutton != 'false') {
                        return new Promise((resolve, reject) => {
                            if (r.body == null) var hackRecord = []
                            else var hackRecord = JSON.parse(r.body);
                            var hackRecordData = []
                            if (hackRecord.length) {
                                for (var i = 0; i < hackRecord.length; i++) {
                                    hackRecordData.push({ Name: hackRecord[i].Name, BreachDate: hackRecord[i].BreachDate, Description: hackRecord[i].Description })
                                }
                                return resolve(hackRecordData)
                            } else return resolve(hackRecordData);
                        })
                    } else return true;//throw new RequestError("Coudnt find hack record")
                }).then(r => {
                    if (r) {
                        data['hackRecords'] = r;
                        return data;
                    } else throw new RequestError("Coudnt parse hack Record")
                }).then(r => {
                    if (r) return new TAB().checkSpamHam({ to: req.body.to, from: req.body.from, header: req.body.header, subject: req.body.subject })
                    //return db.aggregate.findOne({where: {reportedBy: req.body.to , senderId: req.body.from}})
                    else return true;//throw new RequestError("coudnt put hackRecord in data")
                }).then(r => {
                    console.log("check spam ham r is", r);
                    // data['spambutton'] = 'spam'
                    if (r.success == "Found") data['spambutton'] = 'spam';
                    else data['spambutton'] = 'ham';
                    return data
                }).then(r => {
                    if (r) return db.mailsReported.findOne({ where: { email: { like: '%' + req.body.from + '%' } } })
                    else throw new RequestError("Coudnt find user spam /ham")
                })
                    .then(r => {
                        if (r) {
                            data['reportedCount'] = r.spamCount;
                            res.status(200).json({ data: data, settings: settings })
                        } else {
                            data['reportedCount'] = 0;
                            res.status(200).json({ data: data, settings: settings })
                        }
                        console.log("If response", data);
                        // client.set(filename, (JSON.stringify({ data: data, settings: settings })), redis.print);
                    }).catch(err => {
                        console.log(err);
                        next(err);
                    })
            })
    },

    async checkConfig(req, res, next) {
        new Dashboard().checkConfig({ to: req.query.to })
            .then(r => {
                if (r) res.status(200).json({ success: r.data })
                else throw new RequestError("coudnt find Any configuration for Reported mails from dashboard")
            })
            .catch(err => {
                console.log(err);
                next(err)
            })
    },

    async parseEmal64(req, res, next) {
        console.log("REquest body for parseEmail64", req.body.to);
        var filename = makeid() + '.eml';
        var filePath = path.join(appRoot, '..', '..', 'Content', 'Unprocessed', filename);
        // console.log("filepath", filePath)
        return new Promise((resolve, reject) => {
            fs.writeFile(filePath, req.body.body, '', function (err) {
                if (err) reject(err)
                resolve(req.body.body)
            })
        }).then(r => {
            if (r) {
                return new Promise((resolve, reject) => {
                    fs.readFile(filePath, 'utf8', function (err, data) {
                        // console.log("Err", err);
                        // console.log("data",data);
                        if (err) throw new RequestError("Cannot convert File to readable Format")
                        resolve(data)
                    })
                })
            } else throw new RequestError("Cannot convert Base 64 to EML file ")
        }).then(r => {
            // console.log("R", r);
            if (r) {
                new Promise((resolve, reject) => {
                    emlformat.read(r, function (RequestError, data) {
                        // console.log("RequestError while reading emlformat", RequestError)
                        // console.log("data", data)
                        if (RequestError) return reject(RequestError);
                        var body = data.html ? data.html : data.text;
                        var attachment = [];
                        if (!(data.attachments == undefined)) {
                            for (var i = 0; i < data.attachments.length; i++) {
                                console.log("data.attachments[String(i)].contentType", data.attachments[String(i)].contentType, data.attachments[i].data)
                                if (data.attachments[String(i)].name && data.attachments[String(i)]) {
                                    attachment.push({
                                        Key: data.attachments[String(i)].name,
                                        Value: data.attachments[i].data
                                    });
                                } else if (data.attachments[String(i)].contentType && data.attachments[String(i)] && data.attachments[String(i)].contentType.split('name=')[1]) {
                                    attachment.push({
                                        Key: data.attachments[String(i)].contentType.split('name=')[1].split('"')[1],
                                        Value: data.attachments[i].data
                                    });
                                } else if (data.attachments[String(i)].contentType && data.attachments[String(i)] && data.attachments[String(i)].contentType.split('boundary=')[1]) {
                                    console.log("data.attachments[String(i)].contentType", data.attachments[String(i)].contentType.split('boundary=')[1].split('"')[1], data.attachments[i].data)
                                    attachment.push({
                                        Key: data.attachments[String(i)].contentType.split('boundary=')[1].split('"')[1],
                                        Value: data.attachments[i].data
                                    })
                                }
                            };
                        } else {
                            console.log("No Attachments");
                            attachment = []
                        };
                        // console.log("attachment",data);
                        if (body && attachment) {
                            resolve({ body: body, attachments: attachment, from: data.from ? data.from.email : "", to: data.to ? (data.to.email ? data.to.email : req.body.to) : data["Delivered-To"] })
                        } else reject("Coudnt not parse eml")
                    });
                })
                    .then(r => {
                        if (r) {
                            fs.unlink(filePath, (err) => {
                                if (err) reject("Coundt Delet EML File")
                            })
                            return r;
                        } else throw new RequestError("Coudnt Parse EML")
                    })
                    .then(r => {
                        res.status(200).json({ result: r })
                    })
                    .catch(err => {
                        console.log(err, "RequestErrorRR!!")
                        res.status(500)
                        throw new RequestError("Cannot read file to convert to string")
                    })
            } else throw new RequestError("Cannot read file to convert to string")
        }).catch(err => {
            console.log(err);
            res.status(500)
        })
    },

    async parseEmal64Options(req, res, next) {
        res.header("Access-Control-Allow-Origin", "*");
        res.header("Access-Control-Allow-Headers", "Origin, X-Requested-With, Content-Type,Accept");
        res.sendStatus(204)
    },
    async getHeaders(req, res, next) {
        // console.log("Inside getHeaders", req.body);
        let headers = await makeRequestForOwa(req.body.url, req.body.token, 'TAB/1.0', "GET");
        let statusCode = 200;
        if (headers && headers.indexOf("MIME-Version") == -1 && headers.indexOf("Received:") == -1)
            headers = "Some RequestError occured. Kindly contact support";
        else if (headers && isJSON(headers)) {
            headers = JSON.parse(headers)
            statusCode = 500;
            if (headers["RequestError"] && headers.RequestError.code.indexOf("Denied") != -1)
                headers = "EWS is restricted in your environment";
            else
                headers = "Some RequestError occured. Kindly contact support.";
        }
        res.status(statusCode).send(headers);
    },
    async moveMail(req, res, next) {
        console.log("Inside moveMail", req.body);
        let destination = { "DestinationId": "junkEmail" };
        if (req.body.destination)
            destination = { "DestinationId": req.body.destination }
        await makeRequestForOwa("https://outlook.office365.com/api/v2.0/me/messages/" + req.body.itemId, req.body.token, 'TAB/1.0', "PATCH", { "Categories": ["Red Category", "Suspicious Mail(TAB)"] }); //Add label to mail
        let move = await makeRequestForOwa(req.body.url, req.body.token, 'TAB/1.0', "POST", destination);
        console.log("Mail moved", move);
        res.status(200).send(move)
    }
}