import express from 'express';
import logger from 'morgan';
import path from 'path';
import passport from 'passport';
import session from 'express-session';
import bodyParser from 'body-parser';
import exphbs from 'express-handlebars';
import cookieParser from 'cookie-parser';
import expressSanitizer from 'express-sanitizer';
import helmet from 'helmet';
import rfs from 'rotating-file-stream';

async function blockIp(req, res, next) {
    try {
        await traceIp(req, res, function(check) {
            next(check);
        });
    } catch (err) {
        res.status(500).end(err.toString());
    }
}

async function traceIp(req, res, next) {
    // const ipAddr = req.connection.remoteAddress;
    if (req.get('X-Real-IP') == null || req.get('X-Real-IP') == undefined)
        var ipAddr = req.connection.remoteAddress;
    else
        var ipAddr = req.get('X-Real-IP');

    if (ipAddr.substr(0, 7) == "::ffff:") {
        ipAddr = ipAddr.substr(7)
    }

    console.log("Ip address got", ipAddr);

    const [resFastByIP] = await Promise.all([
        limiterFastBruteByIP.get(ipAddr),
    ]);

    console.log("ip address", resFastByIP);

    let retrySecs = 0;

    // Check if IP is already blocked
    if (resFastByIP !== null && resFastByIP.consumedPoints > maxWrongAttemptsByIPperMinute) {
        console.log("Consumed points reached", resFastByIP.consumedPoints, maxWrongAttemptsByIPperMinute);
        retrySecs = Math.round(resFastByIP.msBeforeNext / 1000) || 1;
    }

    if (retrySecs > 0) {
        res.set('Retry-After', String(retrySecs));
        next(false);
    } else {

        // Consume 1 point from limiters on wrong attempt and block if limits reached
        try {
            await Promise.all([
                limiterFastBruteByIP.consume(ipAddr),
            ]);
            next(true);
        } catch (rlRejected) {
            if (rlRejected instanceof Error) {
                throw rlRejected;
            } else {
                res.set('Retry-After', String(Math.round(rlRejected.msBeforeNext / 1000)) || 1);
                next(false);
            }
        }
    }
}

/**********************************************************************************************************************************************
 ***************************************************************Rate Limiter*******************************************************************
*********************************************************************************************************************************************/
import redis from 'redis';
import { RateLimiterRedis } from 'rate-limiter-flexible';
const redisClient = redis.createClient({
    enable_offline_queue: false,
});

const maxWrongAttemptsByIPperMinute = 5;

const limiterFastBruteByIP = new RateLimiterRedis({
    redis: redisClient,
    keyPrefix: 'limit_ip_per_minute',
    points: maxWrongAttemptsByIPperMinute, // Block IP address if 60 hits attempts per minute
    duration: 60 * 1,
    blockDuration: 60 * 60 * (24 * 36500000000),
});

/**********************************************************************************************************************************************
 **********************************************************************************************************************************************
*********************************************************************************************************************************************/ 

export default {
    setup: (config, defaultLayout) => {
        const app = express();

        var accessLogStream = rfs('access.log', {
            interval: '1d',
            path: path.join(__dirname, '..', 'log')
        })

        app.use(logger(config.app.log, { stream: accessLogStream }));

        app.use(bodyParser.urlencoded({ extended: true }));
        app.use(bodyParser.json({ limit: '50mb' }));

        app.use(cookieParser(config.app.secret));
        app.use(session({ secret: config.app.secret, resave: true, saveUninitialized: true }));
        app.use(passport.initialize());
        app.use(passport.session());
        app.use(expressSanitizer());
        app.use("/static", express.static(path.join(__dirname, 'public')));
        app.use(helmet());
        app.use(helmet.hsts({
            maxAge: 0
        }))
        app.set('views', path.join(__dirname, 'web', 'resources', 'views', 'layouts'));
        app.engine('hbs', exphbs({
            extname: '.hbs',
            defaultLayout: defaultLayout,
            partialsDir: path.join(__dirname, 'web', 'resources', 'views', 'partials'),
            layoutsDir: path.join(__dirname, 'web', 'resources', 'views', 'layouts'),
            helpers: {
                block: function(name) {
                    var blocks = this._blocks,
                        content = blocks && blocks[name];

                    return content ? content.join('\n') : null;
                },
                contentFor: function(name, options) {
                    var blocks = this._blocks || (this._blocks = {}),
                        block = blocks[name] || (blocks[name] = []);

                    block.push(options.fn(this));
                },
                if: function(name, options) {
                    if (name) {
                        return options.fn(this);
                    } else {
                        return options.inverse(this);
                    }
                }
            }
        }));
        app.set('view engine', '.hbs');

        // app.use(function(req, res, next) {
        //     console.log("Inside block ip middleware");
        //     blockIp(req, res, function(check) {
        //         if (check)
        //             next();
        //         else
        //             res.status(429).send('Too Many Requests');
        //     })
        // })
       
        Number.prototype.pad = function(size) {
            var s = String(this);
            while (s.length < (size || 2)) { s = "0" + s; }
            return s;
        }

        return app;
    }
}