import passport from 'passport';
import { Strategy as JwtStrategy } from 'passport-jwt';
import { Strategy as LocalStrategy } from 'passport-local';
import bcrypt from 'bcrypt-nodejs';
import config from './config';
import { db } from './models';

var TokenExtractor = function(req) {
    var token = null;
    if (req && req.cookies) {
        token = req.cookies['XSRF-token-tab'];
    }
    return token;
}


// JSON WEB TOKENS STRATEGY
passport.use(new JwtStrategy({
    jwtFromRequest: TokenExtractor,
    secretOrKey: config.app.secret,
}, async(payload, done) => {
    try {
        // console.log("Inside passport",payload.sub)
        var user = null;
        if (payload.iam == "employee") user = await db.Employee.findOne({ where: { id: payload.sub }, required: true, include: [{ model: db.Company, include: [{ model: db.MetaAuth }] }] });
        else if (payload.iam == "root") {
            user = await db.User.findOne({ where: { id: payload.sub }, required: true, include: [{ model: db.Company, include: [{ model: db.MetaAuth }] }] });
        }
        user.type = payload.iam;
        const tokenDate = new Date(payload.iat);
        if (user.loggedOutAt != null && (tokenDate.getTime() - user.loggedOutAt.getTime()) < 0) {
            return done('invalid', false);
        }

        if (new Date(payload.exp) < new Date()) {
            return done('expired', false);
        }
        user.type = payload.iam

        if (user.isNewUser) {
            return done(null, user, 'isNewUser/' + user.email);
        }
        if (!user) {
            return done('user', false);
        }
        // console.log("user found",JSON.parse(JSON.stringify(user)));
        done(null, user);
    } catch (error) {
        done(error, false);
    }
}));

// LOCAL STRATEGY
passport.use(new LocalStrategy({
    usernameField: 'email',
    passReqToCallback: true
}, async(req, email, password, done) => {
    try {
        const alias = req.body.alias;
        db.Employee.findOne({ where: { email: email }, attributes: ['id', 'firstname', 'lastname', 'email', 'password', 'attempt', 'loggedOutAt', 'status', 'valid', 'createdAt', 'isNewUser', 'profileImage'], required: true, include: [{ model: db.Company, where: { $or: { alias: { like: alias }, key: { like: alias } } } }] })
            .then(async employee => {
                if (!employee) {
                    return done(null, false);
                }

                if (employee.attempt == 5) {
                    return done('attempt', false);
                }
                var isMatch = await employee.comparePassword(password)
                    //    var isMatch=  bcrypt.compareSync(password, employee.password);
                if (!isMatch) {
                    employee.update({
                        attempt: employee.attempt + 1
                    })
                    return done('attempt:' + (5 - employee.attempt), false);
                } else if (employee.isNewUser) {
                    return done('isNewUser/' + email, false);
                } else {
                    employee.update({
                        attempt: 0
                    })
                }
                if (!employee.valid) {
                    return done('invalid', false);
                }

                done(null, employee);
            })
    } catch (error) {
        done(error, false);
    }
}));

// IAM ROOT STRATEGY
passport.use('root-login', new LocalStrategy({
    usernameField: 'email'
}, async(email, password, done) => {
    try {
        const user = await db.User.findOne({
            where: {
                email: email
            },
            attributes: ['id', 'firstname', 'lastname', 'email', 'password', 'attempt', 'loggedOutAt', 'status', 'valid', 'createdAt', 'isNewUser', 'companyId', 'twoFA', 'profileImage']
        });
        // if (!user) {
        //     return done("Login failed! Either TWO Factor is enabled or authentication failure!", false);
        // }

        if (user.attempt == 5) {
            return done('attempt', false);
        }
        // var isMatch = await db.User.comparePassword(password)
        var isMatch = bcrypt.compareSync(password, user.password);

        if (!isMatch) {
            user.update({
                attempt: user.attempt + 1
            })
            return done('attempt:' + (5 - user.attempt), false);
        } else {

            user.update({ attempt: 0 })
        }

        if (!user.valid) {
            return done('invalid', false);
        }

        done(null, user);
    } catch (error) {
        done(error, false);
    }
}));


// LOCAL STRATEGY
passport.use('magic-login', new LocalStrategy({
    usernameField: 'email',
    passReqToCallback: true
}, async(req, email, password, done) => {
    try {
        const { email, password } = req.params;
        const user = await db.User.findOne({ where: { email: email, isNewUser: 1 }, attributes: ['id', 'firstname', 'lastname', 'email', 'password', 'attempt', 'loggedOutAt', 'status', 'valid', 'createdAt', 'isNewUser', 'profileImage'] });
        if (!user) {
            return done(null, false);
        }

        var isMatch = await user.comparePassword(password)

        if (!isMatch) {
            return done('notMatch', false);
        } else {
            user.update({ attempt: 0 })
        }
        user.type = "root";
        done(null, user);
    } catch (error) {
        done(error, false);
    }
}));


// LOCAL STRATEGY
passport.use('otp-login', new LocalStrategy({
    usernameField: 'email'
}, async(email, password, done) => {
    try {
        const user = await db.User.findOne({ where: { email: email }, attributes: ['id', 'firstName', 'lastName', 'phone', 'email', 'password', 'otp', 'attempt', 'loggedOutAt', 'status', 'valid', 'createdAt', 'isNewUser', 'profileImage'] });
        if (!user) {
            return done(null, false);
        }

        if (user.attempt == 5) {
            return done('attempt', false);
        }
        var isMatch = (user.otp == password)

        if (!isMatch) {
            user.update({ attempt: user.attempt + 1 })
            return done('attempt:' + (5 - user.attempt), false);
        } else {
            user.update({ attempt: 0 })
        }
        const date = new Date().getTime();
        var expiry = (date >= user.otpExpiry);
        if (expiry) {
            return done('expired', false);
        }
        // if (!user.valid) {
        //     return done('invalid', false);
        // }
        done(null, user);
    } catch (error) {
        done(error, false);
    }
}));

//LOCAL STRATEGY
passport.use('Employee-magic-login', new LocalStrategy({
    usernameField: 'email',
    passReqToCallback: true
}, async(req, email, password, done) => {
    try {
        const { email, password, alias } = req.params;
        const user = await db.Employee.findOne({ where: { email: email }, attributes: ['id', 'firstname', 'lastname', 'email', 'password', 'otp', 'otpExpiry', 'attempt', 'loggedOutAt', 'status', 'valid', 'createdAt', 'isNewUser', 'profileImage'], required: true, include: [{ model: db.Company, where: { $or: { alias: { like: alias }, key: { like: alias } } } }] })
        if (!user) {
            return done(null, false);
        }
        var isMatch = (user.otp == password);
        if (!isMatch) {
            return done('notMatch', false);
        } else {
            user.update({ attempt: 0 })
        }
        var date = new Date().getTime();
        var expiry = (date >= user.otpExpiry);
        if (expiry) {
            return done('expired', false);
        } else {
            user.update({ otpExpiry: 0 })
        }

        user.type = "employee"
        done(null, user);
    } catch (error) {
        done(error, false);
    }
}));

passport.use('Employee-reset-password-magic-login', new LocalStrategy({
    usernameField: 'email',
    passReqToCallback: true
}, async(req, email, password, done) => {
    try {
        const { email, password, alias } = req.params;
        const user = await db.Employee.findOne({ where: { email: email }, attributes: ['id', 'firstname', 'lastname', 'email', 'password', 'otp', 'otpExpiry', 'attempt', 'loggedOutAt', 'status', 'valid', 'createdAt', 'isNewUser', 'profileImage'], required: true, include: [{ model: db.Company, where: { $or: { alias: { like: alias }, key: { like: alias } } } }] })
        if (!user) {
            return done(null, false);
        }
        var isMatch = bcrypt.compareSync(password, user.password);

        if (!isMatch) {
            return done('notMatch', false);
        } else {
            user.update({ attempt: 0 })
        }
        user.type = "employee"
        done(null, user);
    } catch (error) {
        done(error, false);
    }
}));

passport.use('jwt-threatcop', new LocalStrategy({
    usernameField: 'email'
}, async(email, password, done) => {
    try {
        const user = await db.User.findOne({
            where: {
                email: email
            },
            attributes: ['id', 'firstname', 'lastname', 'email', 'password', 'attempt', 'loggedOutAt', 'status', 'valid', 'createdAt', 'isNewUser', 'companyId', 'twoFA', 'profileImage']
        });
        if (!user) {
            return done(null, false);
        }
        done(null, user);
    } catch (error) {
        done(error, false);
    }
}));