import fs from 'fs';
import kue from 'kue';
import request from 'request';
import path from 'path';
import { db } from './models';
import config from './config';
import VirusTotal from './modules/VirusTotal';
import Tab from './modules/Tab';
// import VirusTotalApi from 'virustotal-api';


export var queue = kue.createQueue({
	prefix: 'scanner',
	redis: {
		host: config.redis.host,
		port: config.redis.port,
		auth: config.redis.password
	}
});

var detected = function (result) {
	var content = JSON.parse(result);
	if (content) {
		var keys = Object.entries(content.scans);
		var detectedTrue = 0, virusCountTotal = 0
		for (var i = 0; i < keys.length; i++) {
			virusCountTotal = virusCountTotal + 1;
			if (keys[i][1].detected == true) {
				detectedTrue = detectedTrue + 1
			}
		}
		return [detectedTrue, virusCountTotal]
	}
}

export default {
	init: () => {
		queue.process('virus-scan-file', (job, done) => {
			console.log('virus scanning started', job.data);
			var fileName = job.data.name;
			var formData = {
				apiKey: job.data.apiKey,
				file: fs.createReadStream(path.join(appRoot, '..', '..', 'Content', 'Attachments', fileName))
			};
			new VirusTotal().scanFile(formData)
				.then(([data, code]) => {
					if (code == 200 && data) {
						// data = JSON.parse(data)
						return db.VirusTotal.create({
							scanId: data.resource,
							status: 'pending',
							contentId: job.data.contentId,
						}).then(r => [r, data])
					} else throw new Error(`Unexpected error with code ${code}`)
				})
				.then(([r, data]) => {
					console.log('File successfully went for scanning', job.data.name);
					var resource = { 'scanId': data.resource, 'id': job.data.contentId };
					queue.create('attachment-scan-result', { resource: resource, apiKey: job.data.apiKey, fileName: job.data.name }).save();
					done();
				})
				.catch(err => {
					console.log(err);
					done(err);
				})
		})

		queue.process('attachment-scan-result', (job, done) => {
			new VirusTotal().reportAttachment({ resource: job.data.resource.scanId, apikey: job.data.apiKey })
				.then(([body, statusCode]) => {
					if (statusCode == 204 || (statusCode == 200 && body && body['response_code'] == -2)) {
						queue.create('VirusTotal-Delay', { 'type': 'attachment-scan-result', 'data': { resource: job.data.resource, apikey: job.data.apiKey } }).save();
						throw new Error("Come back later for result", 2044)
					}
					else if (statusCode == 200 && body) return body;
					throw new Error(`Virus total retured with unexpected status code ${statusCode}`, statusCode)
				})
				.then(body => {
					var status = (body['response_code'] == -2) ? 'pending' : 'completed';
					return db.VirusTotal.update({
						status: status,
						result: JSON.stringify(body),
						virusCount: body.positives,
						totalScan: body.total
					}, {
						where: {
							id: job.data.resource.id,
						}
					}).then(r => [r, body])
				})
				.then(([r, body]) => {
					return new Tab().gctxScore({ fileName: job.data.fileName, score: body.positives, total: body.total, type: "Attachment" })
						.then(r => {
							done();
						})
				})
				.catch(err => {
					console.log('kue', 'attachment-scan-result', err);
					done(err)
				})
		})

		queue.process('url-scan-result', (job, done) => {
			new VirusTotal().getUrlResult({ resource: job.data.resource.scanId, apikey: job.data.apikey })
				.then(([content, statusCode]) => {
					if (statusCode == 200) {
						if (content && content['response_code'] == -2) {
							throw new Error('Scan result not yet available');
						} else if (content) {
							var result = JSON.stringify(content);
							var ratio = detected(result)
							var status = 'completed';
							return db.VirusTotal.update({
								status: status,
								result: result,
								virusCount: ratio[0],
								totalScan: ratio[1]
							}, {
								where: {
									id: job.data.resource.id,
								}
							}).then(r => [r, content])// update TAB api to update Status 
						} else throw new Error('Invalid Request')
					} else if (statusCode == 204) {
						console.log("Scan Failed due to VirusToal Limit Exceed for - Retrying")
						queue.create('VirusTotal-Delay', { 'type': 'url-scan-result', 'data': { resource: job.data.resource, apikey: job.data.apiKey } }).save();
					}
				})
				.then(([r, content]) => {
					return new Tab().gctxScore({ urlName: job.data.urlName, score: content.positives, total: content.total, type: "Url" })
						.then(r => {
							done();
						})
				})
				.catch(err => {
					console.log(err)
					done(err)
				})
		})


		queue.process('virus-scan-url', (job, done) => {
			var formData = {
				apiKey: job.data.apiKey,
				url: job.data.name,
			};
			new VirusTotal().reportUrl(formData)
				.then(([body, statusCode]) => {
					if (statusCode == 200) {
						var scanID = body.scan_id
						return db.VirusTotal.findOne({
							where: { scanId: scanID }
						}).then(r => [r, scanID])
					}
					throw new Error(statusCode)
				})
				.then(([r, scanId]) => {
					if (!r) return db.VirusTotal.create({
						scanId: scanId,
						status: "pending",
						contentId: job.data.contentId,
					}).then(r => [r, scanId])
					return [r, scanId];
				})
				.then(([r, scanId]) => {
					console.log('Successfully queued URL for scanning', job.data.name);
					var resource = { 'scanId': scanId, 'id': job.data.contentId };
					queue.create('url-scan-result', { resource: resource, apikey: job.data.apiKey, urlName: job.data.name }).save();
					done();
				})
				.catch(err => {
					if (err.message == 204) {
						console.log("Scan Failed due to VirusToal Limit Exceed")
						queue.create('VirusTotal-Delay', { 'type': 'virus-scan-url', 'data': { 'name': job.data.name, 'type': job.data.type, 'contentId': job.data.contentId, 'apiKey': job.data.apiKey } }).save();
						done();
					} else {
						console.log('Scan failed.', err)
						db.VirusTotal.create({
							scanId: null,
							status: "failed",
							contentId: job.data.contentId,
							result: JSON.stringify(err)
						})
							.then(r => done(err))
							.catch(err1 => done(err))
					}
				})
		})

		queue.process('VirusTotal-Delay', (job, done) => {
			setTimeout(function () {
				queue.create(job.data.type, job.data.data).save();
				done();
			}, 20000)
		})
	}
}